<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 08/29/2004 *
 * ===============                              Last change: 08/29/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : lib-mysql3.php                                   *
 * -------------------------------------------------------------------- *
 * Short description : Database layer for MySQL +3.x server             *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Datenbankschicht fuer MySQL +3.x Server          *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * Needs to be in all Files and every File needs "svn propset           *
 * svn:keywords Date Revision" (autoprobset!) at least!!!!!!            *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2008 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	$INC = substr(dirname(__FILE__), 0, strpos(dirname(__FILE__), '/inc') + 4) . '/security.php';
	require($INC);
}

// SQL queries
function SQL_QUERY ($sql_string, $F, $L) {
	// Link is up?
	if (!SQL_IS_LINK_UP()) {
		// We should not quietly ignore this!
		trigger_error(sprintf("Cannot query database: sql_string=%s,file=%s,line=%s",
			$sql_string,
			basename($F),
			$L
		));

		// Return 'false' because it has failed
		return false;
	} // END - if

	// Remove \t, \n and \r from queries they may confuse some MySQL version I have heard
	$sql_string = str_replace("\t", " ", str_replace("\n", " ", str_replace("\r", " ", $sql_string)));

	// Replace {!_MYSQL_PREFIX!} with constant, closes #84. Thanks to profi-concept
	$sql_string = str_replace("{!_MYSQL_PREFIX!}", getConfig('_MYSQL_PREFIX'), $sql_string);

	// Replace {!_TABLE_TYPE!} with constant
	$sql_string = str_replace("{!_TABLE_TYPE!}", getConfig('_TABLE_TYPE'), $sql_string);

	// Starting time
	$querytimeBefore = array_sum(explode(' ', microtime()));

	// Run SQL command
	//* DEBUG: */ echo $sql_string."<br />\n";
	$result = mysql_query($sql_string, SQL_GET_LINK())
		or addFatalMessage(__FUNCTION__, __LINE__, $F." (".$L."):".mysql_error()."<br />
Query string:<br />
".$sql_string);

	// Ending time
	$querytimeAfter = array_sum(explode(' ', microtime()));

	// Calculate query time
	$queryTime = $querytimeAfter - $querytimeBefore;

	// Save last successfull query
	setConfigEntry('db_last_query', $sql_string);

	// Count this query
	incrementConfigEntry('sql_count');

	// Debug output
	//* DEBUG: */ print "Query=<pre>".$sql_string."</pre>, affected=<strong>".SQL_AFFECTEDROWS()."</strong>, numrows=<strong>".SQL_NUMROWS($result)."</strong><br />\n";
	if ((getOutputMode() != '1') && (getOutputMode() != '-1') && (isDebugModeEnabled()) && (getConfig('DEBUG_SQL') == 'Y')) {
		//
		// Debugging stuff...
		//
		$fp = fopen(constant('PATH') . 'inc/cache/mysql.log', 'a') or app_die(__FILE__, __LINE__, "Cannot write mysql.log!");
		if (!isset($GLOBALS['sql_first_entry'])) {
			// Write first entry
			fwrite($fp, 'Module=' . getModule() . "\n");
			$GLOBALS['sql_first_entry'] = true;
		} // END - if
		fwrite($fp, $F."(LINE=".$L."|NUM=".SQL_NUMROWS($result)."|AFFECTED=".SQL_AFFECTEDROWS()."|QUERYTIME:".$queryTime."): ".str_replace("\r", '', str_replace("\n", " ", $sql_string))."\n");
		fclose($fp);
	} // END - if

	// Count DB hits
	if (!isConfigEntrySet('db_hits_run')) {
		// Count in dummy variable
		setConfigEntry('db_hits_run', 1);
	} else {
		// Count to config array
		incrementConfigEntry('db_hits_run');
	}

	// Return the result
	return $result;
}

// SQL num rows
function SQL_NUMROWS ($result) {
	// Is the result a valid resource?
	if (is_resource($result)) {
		// Get the count of rows from database
		$lines = mysql_num_rows($result);

		// Is the result empty? Then we have an error!
		if (empty($lines)) $lines = 0;
	} else {
		// No resource given, no lines found!
		$lines = 0;
	}
	return $lines;
}

// SQL affected rows
function SQL_AFFECTEDROWS() {
	// Valid link resource?
	if (!SQL_IS_LINK_UP()) return false;

	// Get affected rows
	$lines = mysql_affected_rows(SQL_GET_LINK());

	// Return it
	return $lines;
}

// SQL fetch row
function SQL_FETCHROW ($result) {
	// Is a result resource set?
	if (!is_resource($result)) return false;

	// Fetch the data and return it
	return mysql_fetch_row($result);
}

// SQL fetch array
function SQL_FETCHARRAY($res, $nr=0, $remove_numerical=true) {
	// Is a result resource set?
	if (!is_resource($res)) return false;

	// Initialize array
	$row = array();

	// Load row from database
	$row = mysql_fetch_array($res);

	// Return only arrays here
	if (is_array($row)) {
		// Shall we remove numerical data here automatically?
		if ($remove_numerical) {
			// So let's remove all numerical elements to save memory!
			$max = count($row);
			for ($idx = 0; $idx < ($max / 2); $idx++) {
				// Remove entry
				unset($row[$idx]);
			} // END - for
		} // END - if

		// Return row
		return $row;
	} else {
		// Return a false here...
		return false;
	}
}

// SQL result
function SQL_RESULT ($res, $row, $field = 0) {
	// Is $res valid?
	if (!is_resource($res)) return false;

	// Run the result command and return the result
	$result = mysql_result($res, $row, $field);
	return $result;
}

// SQL connect
function SQL_CONNECT ($host, $login, $password, $F, $L) {
	// Try to connect
	$connect = mysql_connect($host, $login, $password) or addFatalMessage(__FUNCTION__, __LINE__, $F." (".$L."):".mysql_error());

	// Set the link resource
	SQL_SET_LINK($connect);
}

// SQL select database
function SQL_SELECT_DB ($dbName, $F, $L) {
	// Is there still a valid link? If not, skip it.
	if (!SQL_IS_LINK_UP()) return false;

	// Return the result
	return mysql_select_db($dbName, SQL_GET_LINK()) or addFatalMessage(__FUNCTION__, __LINE__, $F." (".$L."):".mysql_error());
}

// SQL close link
function SQL_CLOSE ($F, $L) {
	if (!SQL_IS_LINK_UP()) {
		// Skip double close
		return false;
	} // END - if

	// Do we need to update cache/db counter?
	//* DEBUG: */ echo "DB=".getConfig('db_hits').",CACHE=".getConfig('cache_hits')."<br />\n";
	if ((GET_EXT_VERSION('cache') >= '0.0.7') && (getConfig('db_hits') > 0) && (getConfig('cache_hits') > 0) && (isCacheInstanceValid())) {
		// Add new hits
		incrementConfigEntry('db_hits', getConfig('db_hits_run'));

		// Update counter for db/cache
		updateConfiguration(array('db_hits', 'cache_hits'), array(getConfig('db_hits'), getConfig('cache_hits')));
	} // END - if

	// Close database link and forget the link
	$close = mysql_close(SQL_GET_LINK())
		or addFatalMessage(__FUNCTION__, __LINE__, $F . ' (' . $L . '):'.mysql_error());

	// Close link
	SQL_SET_LINK(null);

	// Return the result
	return $close;
}

// SQL free result
function SQL_FREERESULT ($result) {
	if (!is_resource($result)) {
		// Abort here
		return false;
	} // END - if

	$res = mysql_free_result($result);
	return $res;
}

// SQL string escaping
function SQL_QUERY_ESC ($qstring, $data, $F, $L, $run=true, $strip=true, $secure=true) {
	// Link is there?
	if (!SQL_IS_LINK_UP()) return false;

	// Init variable
	$query = 'failed';

	if ($strip === true) {
		$strip = 'true';
	} else {
		$strip = 'false';
	}

	if ($secure === true) {
		$secure = 'true';
	} else {
		$secure = 'false';
	}

	$eval = "\$query = sprintf(\"".$qstring."\"";
	foreach ($data as $var) {
		if ((!empty($var)) || ($var === 0)) {
			$eval .= ", SQL_ESCAPE(\"".$var."\", ".$secure.", ".$strip.")";
		} else {
			$eval .= ", ''";
		}
	} // END - foreach
	$eval .= ");";

	// Debugging
	//
	//* DEBUG: */ $fp = fopen(constant('PATH')."inc/cache/escape_debug.log", 'a') or app_die(__FILE__, __LINE__, "Cannot write debug.log!");
	//* DEBUG: */ fwrite($fp, $F.'('.$L."): ".str_replace("\r", '', str_replace("\n", " ", $eval))."\n");
	//* DEBUG: */ fclose($fp);

	// Run the code
	eval($eval);

	// Was the eval() command fine?
	if ($query == 'failed') {
		// Something went wrong?
		debug_report_bug('eval=' . $eval);
	} // END - if

	if ($run === true) {
		// Run SQL query (default)
		return SQL_QUERY($query, $F, $L);
	} else {
		// Return secured string
		return $query;
	}
}

// Get ID from last INSERT command
function SQL_INSERTID () {
	if (!SQL_IS_LINK_UP()) return false;
	return mysql_insert_id();
}

// Escape a string for the database
function SQL_ESCAPE ($str, $secureString=true, $strip=true) {
	// Secure string first? (which is the default behaviour!)
	if ($secureString) {
		// Then do it here
		$str = secureString($str, $strip);
	} // END - if

	if (!SQL_IS_LINK_UP()) {
		// Fall-back to smartAddSlashes() when there is no link
		return smartAddSlashes($str);
	} elseif (function_exists('mysql_real_escape_string')) {
		// The new and improved version
		//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):str={$str}<br />\n";
		return mysql_real_escape_string($str, SQL_GET_LINK());
	} elseif (function_exists('mysql_escape_string')) {
		// The obsolete function
		return mysql_escape_string($str, SQL_GET_LINK());
	} else {
		// If nothing else works, fall back to smartAddSlashes()
		return smartAddSlashes($str);
	}
}

// SELECT query string from table, columns and so on... ;-)
function SQL_RESULT_FROM_ARRAY ($table, $columns, $idRow, $id, $F, $L) {
	// Is columns an array?
	if (!is_array($columns)) {
		// No array
		trigger_error(sprintf("columns is not array. %s!=array", gettype($columns)));
	} // END  - if

	// Prepare the SQL statement
	$sql = "SELECT `".implode("`,`", $columns)."` FROM `{!_MYSQL_PREFIX!}_%s` WHERE `%s`='%s' LIMIT 1";

	// Return the result
	return SQL_QUERY_ESC($sql,
		array(
			$table,
			$idRow,
			bigintval($id),
		), $F, $L
	);
}

// ALTER TABLE wrapper function
function SQL_ALTER_TABLE ($sql, $F, $L) {
	// This is the default result...
	$result = false;

	// Determine index/fulltext/unique word
	$noIndex = (
	(
		strpos($sql, 'INDEX') === false
	) && (
		strpos($sql, 'FULLTEXT') === false
	) && (
		strpos($sql, 'UNIQUE') === false
	)
	);

	// Extract table name
	$tableArray = explode(" ", $sql);
	$tableName = str_replace('`', '', $tableArray[2]);

	// Shall we add/drop?
	if (((strpos($sql, 'ADD') !== false) || (strpos($sql, 'DROP') !== false)) && ($noIndex === true)) {
		// And column name as well
		$columnName = str_replace('`', '', $tableArray[4]);

		// Get column information
		$result = SQL_QUERY_ESC("SHOW COLUMNS FROM `%s` LIKE '%s'",
			array($tableName, $columnName), __FILE__, __LINE__);

		// Do we have no entry on ADD or an entry on DROP?
		// 123           4       4     3    3      4           4          32    23           4       4     3    3      4            4          321
		if (((SQL_NUMROWS($result) == 0) && (strpos($sql, 'ADD') !== false)) || ((SQL_NUMROWS($result) == 1) && (strpos($sql, 'DROP') !== false))) {
			// Do the query
			//* DEBUG: */ print __LINE__.':'.$sql."<br />\n";
			$result = SQL_QUERY($sql, $F, $L, false);
		} // END - if
	} elseif ((getConfig('_TABLE_TYPE') == 'InnoDB') && (strpos($sql, 'FULLTEXT') !== false)) {
		// Skip this query silently
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, sprintf("Skipped FULLTEXT: sql=%s,file=%s,line=%s", $sql, $F, $L));
	} elseif (!$noIndex) {
		// And column name as well
		$columnName = str_replace('`', '', $tableArray[4]);

		// Is this "UNIQUE" or so? FULLTEXT has been handled the elseif() block above
		if (in_array(strtoupper($columnName), array('INDEX', 'UNIQUE', 'KEY', 'FULLTEXT'))) {
			// Init loop
			$begin = 1; $columnName = ',';
			while (strpos($columnName, ',') !== false) {
				// Use last
				$columnName = str_replace('`', '', $tableArray[count($tableArray) - $begin]);
				//* DEBUG: */ print __LINE__.':'.$columnName."----------------".$begin."<br />\n";

				// Remove brackes
				$columnName = str_replace('(', '', str_replace(')', '', $columnName));
				//* DEBUG: */ print __LINE__.':'.$columnName."----------------".$begin."<br />\n";

				// Continue
				$begin++;
			} // END while
		} // END - if

		// Show indexes
		$result = SQL_QUERY_ESC("SHOW INDEX FROM `%s`",
			array($tableName), __FILE__, __LINE__);

		// Walk through all
		$skip = false;
		while ($content = SQL_FETCHARRAY($result)) {
			// Is it found?
			//* DEBUG: */ print "<pre>".print_r($content, true)."</pre>";
			if (($content['Column_name'] == $columnName) || ($content['Key_name'] == $columnName)) {
				// Skip this query!
				//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, sprintf("Skiped: %s", $sql));
				$skip = true;
				break;
			} // END - if
		} // END - while

		// Free result
		SQL_FREERESULT($result);

		// Shall we run it?
		if (!$skip) {
			// Send it to the SQL_QUERY() function
			//* DEBUG: */ print __LINE__.':'.$sql."<br />\n";
			$result = SQL_QUERY($sql, $F, $L, false);
		} // END - if
	} else {
		// Other ALTER TABLE query
		//* DEBUG: */ print __LINE__.':'.$sql."<br />\n";
		$result = SQL_QUERY($sql, $F, $L, false);
	}

	// Return result
	return $result;
}

// Getter for SQL link
function SQL_GET_LINK () {
	// Init link
	$link = null;

	// Is it in the globals?
	if (isset($GLOBALS['sql_link'])) {
		// Then take it
		$link = $GLOBALS['sql_link'];
	} // END - if

	// Return it
	return $link;
}

// Setter for link
function SQL_SET_LINK ($link) {
	// Is this a resource or null?
	if ((!is_resource($link)) && (!is_null($link))) {
		// This should never happen!
		trigger_error(sprintf("link is not resource or null. Type: %s", gettype($link)));
	} // END - if

	// Set it
	$GLOBALS['sql_link'] = $link;
}

// Checks if the link is up
function SQL_IS_LINK_UP () {
	// Get the link
	$link = SQL_GET_LINK();

	// Return the result
	return (is_resource($link));
}

//
?>
